// Font.cpp: implementation of the CFont class.
//
//////////////////////////////////////////////////////////////////////
// NeHe example used

#include "Font.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CFONT::CFONT()
{

}

void CFONT::Destroy_Font()
{
	glDeleteLists(m_iFontID, 256);							// Delete All 256 Display Lists
}

// --------------------------------------------------------------
// This function creates font with given parameters
// --------------------------------------------------------------
void CFONT::CreateFont(unsigned int lx, unsigned int ly, unsigned int ni, unsigned int space)
{
	float	cx;											// Holds Our X Character Coord
	float	cy;											// Holds Our Y Character Coord

	m_iLengthX = lx;	m_iLengthY = ly;	m_iSpaceChar = space;	m_iNI = ni;

	if(space <= 5)
		space = 6;
	m_iFontID = glGenLists(256);								// Creating 256 Display Lists
	// glBindTexture(GL_TEXTURE_2D, texture[0]);			// Select Our Font Texture
	for (int loop=0; loop<256; loop++)						// Loop Through All 256 Lists
	{
		cx = float(loop%lx)*0.0625f;						// X Position Of Current Character
		cy = float(loop/ly)*0.0625f;						// Y Position Of Current Character

		glNewList(m_iFontID+loop, GL_COMPILE);				// Start Building A List
			glBegin(GL_QUADS);							// Use A Quad For Each Character
				glTexCoord2f(cx, 1-cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2i(0, 0);						// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f, 1-cy-0.0625f);	// Texture Coord (Bottom Right)
				glVertex2i(lx, 0);						// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f, 1-cy);			// Texture Coord (Top Right)
				glVertex2i(lx, ly);						// Vertex Coord (Top Right)
				glTexCoord2f(cx, 1-cy);					// Texture Coord (Top Left)
				glVertex2i(0, ly);						// Vertex Coord (Top Left)
			glEnd();									// Done Building Our Quad (Character)
		glTranslated(space, 0, 0);						// Move To The Right Of The Character
		glEndList();									// Done Building The Display List
	}													// Loop Until All 256 Are Built
}

// --------------------------------------------------------------
// Prints text on screen position (0,0 is bottom left corner - screen is 800x600)
// --------------------------------------------------------------
void CFONT::glPrint(char *text) // Where The Printing Happens (nib - normal, italic, bold)
{
	if(m_FontMaterial.m_id > 0)
	{
		glEnable(GL_TEXTURE_2D);

		glPushMatrix();
		glBindTexture(GL_TEXTURE_2D, m_FontMaterial.m_id);
		glColor3fv(m_FontMaterial.m_Ambient.m_rgba); 

		glDisable(GL_DEPTH_TEST);							// Disable Depth Testing
		glEnable(GL_BLEND);									// Enable Blending
		glBlendFunc(GL_SRC_ALPHA,GL_ONE);					// Type Of Blending To Perform
		glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// Really Nice Perspective Calculations
		glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);				// Really Nice Point Smoothing

		glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
		glPushMatrix();										// Store The Projection Matrix
		glLoadIdentity();									// Reset The Projection Matrix
		glOrtho(0, 800, 0, 600, -1, 1);						// Set Up An Ortho Screen
		glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
		glPushMatrix();										// Store The Modelview Matrix
		glLoadIdentity();									// Reset The Modelview Matrix
		glMultMatrixf(m_ViewMatrix.m_matrix);
		glListBase(m_iFontID - 32 + (128*m_iNI) );				// Choose The Font Set (0 or 1)
		glCallLists(strlen(text), GL_BYTE, text);			// Write The Text To The Screen
		glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
		glPopMatrix();										// Restore The Old Projection Matrix
		glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
		glPopMatrix();										// Restore The Old Projection Matrix
		glBlendFunc(GL_ONE,GL_ZERO);					// Reset to previous settings
		glEnable(GL_DEPTH_TEST);						// (restore)
		glPopMatrix();
	}
}

bool CFONT::Save_Font(char *filename, FILE *f)
{
	FILE *file = NULL;

	if(f == NULL)
	{
		if(filename == NULL)
			return false;
		else
			file = fopen(filename,"w+t");
	}
	else
		file = f;
	
	if(file !=NULL)
	{
		fprintf(file, "NI %d LX %d LY %d SPACE %d\n", m_iNI, m_iLengthX, m_iLengthY, m_iSpaceChar);
		SaveMaterial(&m_FontMaterial, NULL, file);
	}

	if(f == NULL)
		fclose(file);

	return true;
}

bool CFONT::Load_Font(char *filename, FILE *f)
{
	FILE *file = NULL;

	if(f == NULL)
	{
		if(filename == NULL)
			return false;
		else
			file = fopen(filename,"r+t");
	}
	else
		file = f;
	
	if(file !=NULL)
	{
		fscanf(file, "NI %d LX %d LY %d SPACE %d\n", &m_iNI, &m_iLengthX, &m_iLengthY, &m_iSpaceChar);
		if(LoadMaterial(&m_FontMaterial, NULL, file))
			CreateFont(m_iLengthX, m_iLengthY, m_iNI, m_iSpaceChar);
		else
		{
			if(f == NULL)
				fclose(file);
			return false;
		}
	}

	if(f == NULL)
		fclose(file);

	return true;
}
